@@ -13,9 +13,7 @@ require "rabbit/action"
 require "rabbit/html/generator"
 
 module Rabbit
-
   class Canvas
-    
     include Enumerable
     extend Forwardable
 
@@ -56,6 +54,10 @@ module Rabbit
     def_delegators(:@renderer, :progress_background=)
 
     def_delegators(:@renderer, :make_color, :make_layout)
+    def_delegators(:@renderer, :translate_context, :rotate_context)
+    def_delegators(:@renderer, :scale_context, :reflect_context)
+    def_delegators(:@renderer, :shear_context)
+    def_delegators(:@renderer, :save_context, :restore_context)
     def_delegators(:@renderer, :draw_line, :draw_rectangle, :draw_arc)
     def_delegators(:@renderer, :draw_circle, :draw_layout, :draw_pixbuf)
     def_delegators(:@renderer, :draw_arc_by_radius, :draw_circle_by_radius)
@@ -102,6 +104,8 @@ module Rabbit
     def_delegators(:@renderer, :graffiti_color, :graffiti_color=)
     def_delegators(:@renderer, :graffiti_line_width, :graffiti_line_width=)
 
+    def_delegators(:@renderer, :toggle_info_window)
+
     def_delegators(:@renderer, :add_gesture_action)
 
     def_delegators(:@renderer, :post_init_gui)
@@ -112,7 +116,7 @@ module Rabbit
 
     def_delegators(:@renderer, :search_slide, :stop_slide_search, :searching?)
 
-    def_delegators(:@source, :source=, :reset)
+    def_delegators(:@source, :source=, :reset, :base)
 
     attr_reader :logger, :renderer, :last_modified
     attr_reader :comment_source, :actions
@@ -140,6 +144,7 @@ module Rabbit
       @rss_base_uri = true
       @migemo_dictionary_search_path = []
       @migemo_dictionary_name = nil
+      @limit_time = nil
       @use_gl = false
       init_comment(comment_source, comment_encoding)
       clear
@@ -173,8 +178,8 @@ module Rabbit
     end
 
     def attach_to(frame, window)
-      @frame = frame
-      @renderer.attach_to(window)
+      @frame = frame if frame
+      @renderer.attach_to(window) if window
     end
     
     def detach
@@ -536,6 +541,19 @@ module Rabbit
       @index_mode = current_index_mode
     end
 
+    def allotted_time
+      slide = title_slide
+      slide ? slide.allotted_time : nil
+    end
+
+    def start_timer(limit)
+      @limit_time = Time.now + limit
+    end
+
+    def rest_time
+      @limit_time ? @limit_time - Time.now : nil
+    end
+
     private
     def _apply_theme(name, id, &block)
       @theme_name = name if name
@@ -701,24 +719,5 @@ module Rabbit
     def prepare_comment(comment)
       comment.to_s.gsub(/\r?\n/, '')
     end
-   
-  end
-
-  class CommentCanvas < Canvas
-    def quit
-      toggle_comment_view
-    end
-
-    def saved_image_base_name
-      super + "_comment"
-    end
-
-    def filename=(new_filename)
-      if new_filename.nil?
-        super(new_filename)
-      else
-        super(new_filename + "_comment")
-      end
-    end
   end
 end
