@@ -7,8 +7,7 @@ module Rabbit
       include SlideElement
 
       def initialize(title)
-        super(title.text)
-        add_element(title)
+        super
       end
 
       def headline
