@@ -46,8 +46,20 @@ module Rabbit
         @@poppler_available = nil
 
         class << self
+          def available_with_gdk?
+            Gdk.respond_to?(:cairo_available?) and Gdk.cairo_available?
+          end
+
+          def available_with_pango?
+            Pango.respond_to?(:cairo_available?) and Pango.cairo_available?
+          end
+
           def priority
-            100
+            if available_with_gdk? and available_with_pango?
+              100
+            else
+              -100
+            end
           end
         end
 
@@ -72,32 +84,64 @@ module Rabbit
           make_color(color).to_gdk_rgb
         end
 
+        def translate_context(x, y, params={})
+          @context.translate(x, y)
+        end
+
+        def rotate_context(angle, params={})
+          @context.rotate(convert_angle(angle, 0)[0])
+        end
+
+        def scale_context(x, y, params={})
+          @context.scale(x, y)
+        end
+
+        def reflect_context(base, params={})
+          case base
+          when :y
+            matrix = make_matrix(-1, 0, 0,
+                                  0, 1, 0)
+          else
+            matrix = make_matrix(1,  0, 0,
+                                 0, -1, 0)
+          end
+          @context.transform(matrix)
+        end
+
+        def shear_context(x, y, params={})
+          @context.transform(make_matrix(1, x, 0,
+                                         y, 1, 0))
+        end
+
+        def save_context
+          @context.save
+          super
+        end
+
+        def restore_context
+          @context.restore
+        end
+
         def draw_line(x1, y1, x2, y2, color=nil, params={})
           x1, y1 = from_screen(x1, y1)
           x2, y2 = from_screen(x2, y2)
-          color = make_color(color)
           @context.save do
-            set_color(color)
+            set_source(color, params)
             set_line_width(get_line_width(params))
-            @context.stroke do
-              @context.move_to(x1, y1)
-              @context.line_to(x2, y2)
-            end
+            @context.new_path
+            @context.move_to(x1, y1)
+            @context.line_to(x2, y2)
+            apply_cairo_action(false, params)
           end
         end
 
         def draw_rectangle(filled, x, y, w, h, color=nil, params={})
           x, y = from_screen(x, y)
-          color = make_color(color)
           @context.save do
-            set_color(color)
+            set_source(color, params)
             set_line_width(get_line_width(params))
             @context.rectangle(x, y, w, h)
-            if filled
-              @context.fill
-            else
-              @context.stroke
-            end
+            apply_cairo_action(filled, params)
           end
         end
 
@@ -108,15 +152,11 @@ module Rabbit
           y_radius = params[:y_radius] || radius
 
           @context.save do
-            set_color(make_color(color))
+            set_source(color, params)
             set_line_width(get_line_width(params))
             @context.new_path
             @context.rounded_rectangle(x, y, w, h, x_radius, y_radius)
-            if filled
-              @context.fill
-            else
-              @context.stroke
-            end
+            apply_cairo_action(filled, params)
           end
         end
 
@@ -129,22 +169,15 @@ module Rabbit
         def draw_arc_by_radius(filled, x, y, r, a1, a2, color=nil, params={})
           x, y = from_screen(x, y)
           a1, a2 = convert_angle(a1, a2)
-          color = make_color(color)
           @context.save do
-            set_color(color)
+            set_source(color, params)
             set_line_width(get_line_width(params))
             args = [x, y, r, a1, a2]
-            if filled
-              @context.fill do
-                @context.move_to(x, y)
-                @context.arc(*args)
-                @context.close_path
-              end
-            else
-              @context.stroke do
-                @context.arc(*args)
-              end
-            end
+            action, = cairo_action(filled, params)
+            @context.move_to(x, y) unless action == :stroke
+            @context.arc(*args)
+            @context.close_path unless action == :stroke
+            apply_cairo_action(filled, params)
           end
         end
 
@@ -154,28 +187,22 @@ module Rabbit
 
         def draw_polygon(filled, points, color=nil, params={})
           return if points.empty?
-          color = make_color(color)
           @context.save do
-            set_color(color)
+            set_source(color, params)
             set_line_width(get_line_width(params))
             @context.move_to(*from_screen(*points.first))
             points[1..-1].each do |x, y|
               @context.line_to(*from_screen(x, y))
             end
             @context.line_to(*from_screen(*points.first)) unless params[:opened]
-            if filled
-              @context.fill
-            else
-              @context.stroke
-            end
+            apply_cairo_action(filled, params)
           end
         end
 
         def draw_layout(layout, x, y, color=nil, params={})
           x, y = from_screen(x, y)
-          color = make_color(color)
           @context.save do
-            set_color(color)
+            set_source(color, params)
             set_line_width(get_line_width(params))
             @context.move_to(x, y)
             @context.show_pango_layout(layout)
@@ -248,8 +275,42 @@ module Rabbit
           @background = make_color(@background_color)
         end
 
+        def set_source(color, params={})
+          pattern = params[:pattern]
+          set = false
+          if pattern
+            set = set_pattern(pattern)
+            color ||= Color.parse("#fff0") unless set
+          end
+          set_color(make_color(color)) unless set
+        end
+
+        def set_pattern(info)
+          pattern = nil
+          case info[:type]
+          when :radial
+            cx0, cy0, radius0, cx1, cy1, radius1 = info[:base]
+            cx0, cy0 = from_screen(cx0, cy0)
+            cx1, cy1 = from_screen(cx1, cy1)
+            pattern = ::Cairo::RadialPattern.new(cx0, cy0, radius0,
+                                                 cx1, cy1, radius1)
+            info[:color_stops].each do |rgba|
+              pattern.add_color_stop_rgba(*rgba)
+            end
+          when :linear
+            x, y, w, h = info[:base]
+            x, y = from_screen(x, y)
+            pattern = ::Cairo::LinearPattern.new(x, y, w, h)
+            info[:color_stops].each do |rgba|
+              pattern.add_color_stop_rgba(*rgba)
+            end
+          end
+          @context.set_source(pattern) if pattern
+          !pattern.nil?
+        end
+
         def set_color(color)
-          @context.set_source_rgba(color.to_a)
+          @context.set_source_rgba(make_color(color).to_a)
         end
 
         def set_line_width(line_width)
@@ -266,6 +327,30 @@ module Rabbit
         def from_screen(x, y)
           [x.ceil, y.ceil]
         end
+
+        def cairo_action(filled, params={})
+          if params[:clip]
+            [:clip, params[:clip]]
+          elsif filled
+            :fill
+          else
+            :stroke
+          end
+        end
+
+        def apply_cairo_action(filled, params={})
+          action, *other_info = cairo_action(filled, params)
+          @context.send(action)
+          case action
+          when :clip
+            block, = other_info
+            block.call if block
+          end
+        end
+
+        def make_matrix(xx, xy, x0, yx, yy, y0)
+          ::Cairo::Matrix.new(xx, yx, xy, yy, x0, y0)
+        end
       end
     end
   end
