@@ -20,11 +20,42 @@ module Rabbit
         end
       end
 
+      def translate_context(x, y, params={})
+      end
+
+      def rotate_context(angle, params={})
+      end
+
+      def scale_context(x, y, params={})
+      end
+
+      def reflect_context(base, params={})
+      end
+
+      def shear_context(x, y, params={})
+      end
+
+      def save_context
+        if block_given?
+          begin
+            yield
+          ensure
+            restore_context
+          end
+        end
+      end
+
+      def restore_context
+      end
+
       def draw_slide(slide, simulation)
         unless simulation
+          save_context
           draw_rectangle(true, 0, 0, width, height, @background)
         end
         yield
+      ensure
+        restore_context unless simulation
       end
 
       def draw_circle(filled, x, y, w, h, color=nil, params={})
